@@ -184,8 +184,8 @@ void ff_lrts(TreeModel *mod, MSA *msa, GFF_Set *gff, mode_type mode,
       this_scale = d->cdata->params->data[0];
 
       delta_lnl = alt_lnl - null_lnl;
-      if (delta_lnl <= -0.01)
-	die("ERROR ff_lrts: delta_lnl (%f) <= -0.01\n", delta_lnl);
+      if (delta_lnl <= -1)
+	die("ERROR ff_lrts: delta_lnl (%f) <= -1\n", delta_lnl);
       if (delta_lnl < 0) delta_lnl = 0;
     }
 
@@ -231,7 +231,6 @@ void ff_lrts_sub(TreeModel *mod, MSA *msa, GFF_Set *gff, mode_type mode,
                                      data for supertree/subtree case */
 
   /* init ColFitData -- one for null model, one for alt */
-  modcpy->estimate_branchlens = TM_BRANCHLENS_ALL;
   modcpy->subtree_root = NULL;
   d = ff_init_fit_data(modcpy, msa, ALL, NNEUT, FALSE);
   d2 = ff_init_fit_data(mod, msa, SUBTREE, mode, FALSE); 
@@ -421,6 +420,7 @@ void ff_score_tests_sub(TreeModel *mod, MSA *msa, GFF_Set *gff, mode_type mode,
   TreeModel *modcpy = tm_create_copy(mod); /* need separate copy of tree model
                                               with different internal scaling
                                               data for supertree/subtree case */
+  modcpy->subtree_root = NULL;
 
   /* init FeatFitData -- one for null model, one for alt */
   d = ff_init_fit_data(modcpy, msa, ALL, NNEUT, FALSE);
